/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.basher;

import java.util.List;

/** Defines operations for interacting with the task manager.
 * @author Johan Lindquist
 * @version 1.0
 */
public interface TaskManager
{
    /** Adds a <code>Task</code> into a running task manager.  This will include the specified task
     * in the list of available tasks.
     *
     * @param task The task to add to the task manager.
     */
    public void addTask(Task task);

    /** Retrieves the next, randomly choosen task available.
     *
     * @return A task instance or null of no tasks are available
     */
    public Task getNextRandomTask();

    /** Retrieves all active tasks registered with the task manager.
     *
     * @return A list of <code>Task</code> instances.
     */
    public List getTasks();

    /** Retrieves the task identified by the specified name.  This will search both active and removed task lists.
     *
     * @param name The name of the task to retrieve.
     * @return The specified task or null if it can not be found
     */
    public Task getTaskByName(String name);

    /** Retrives the count of the currently registered tasks.
     *
     * @return The number of tasks currently registered.
     */
    public int getNumberOfTasks();

    /** Removes the specified task from the list of active tasks.
     *
     * @param task The task to remove
     */
    public void removeTask(Task task);

    /** Retrieves the list of removed tasks
     *
     * @return The list of <code>Task</code> instances that have been removed
     */
    public List getRemovedTasks();

    /** Returns the number of tasks that have been removed.
     *
      * @return The number of tasks that have been removed.
     */
    public int getNumberOfRemovedTasks();
}
