/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.basher.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.sourceforge.basher.Scheduler;
import net.sourceforge.basher.internal.TaskRunner;
import net.sourceforge.basher.internal.TaskRunnerFactory;
import net.sourceforge.basher.internal.TimedTaskInvoker;
import org.apache.commons.logging.Log;

/** Implementation of the <code>Scheduler</code> interface.
 *
 * @author Johan Lindquist
 * @version 1.0
 */
public class SchedulerImpl implements Scheduler
{
    private TaskRunnerFactory _taskRunnerFactory;
    private TimedTaskInvoker _timedTaskInvoker;

    private int _numberInitialThreads = 10;

    private int _maxThreads = 20;

    private List _threadList;

    private ThreadGroup _threadGroup;

    private Log _logger;
    private boolean _running = false;


    /** {@inheritDoc} */
    public synchronized void addThread()
    {
        checkInitialized();
        addNewThread();
    }

    /** {@inheritDoc} */
    public void addThreads(final int numToAdd)
    {
        checkInitialized();
        _logger.debug("Adding " + numToAdd + " thread(s)");
        for (int i = 0; i < numToAdd; i++)
        {
              addThread();
        }
        _logger.debug(numToAdd + " thread(s) added");
    }

    /** {@inheritDoc} */
    public synchronized void removeThread()
    {
        checkInitialized();

        if (_threadList.size() > 0)
        {
            _logger.debug("Removing thread from active list");
            TaskRunner taskRunner = (TaskRunner) _threadList.remove(0);
            _logger.debug("Thread removed from active list");
            _logger.debug("Stopping thread");
            taskRunner.stopInvoking();
            _logger.debug("Thread stopped");
            _logger.info("Thread removed");
        }
        else
        {
            _logger.warn("No threads to remove");
        }
    }

    /** {@inheritDoc} */
    public void removeAllThreads()
    {
        checkInitialized();

        int threadsToRemove = _threadList.size();
        _logger.debug("Removing " + threadsToRemove + " thread(s)");
        for (int i = 0; i < threadsToRemove; i++)
        {
            removeThread();
        }
        _logger.debug(threadsToRemove+ " thread(s) removed");
    }

    /** {@inheritDoc} */
    public void stop()
    {
        if (!_running)
        {
            throw new IllegalStateException("Already stopped");
        }
        _logger.info("Stopping scheduler");

        if (_timedTaskInvoker.isRunning())
        {
            _timedTaskInvoker.stop();
        }

        for (Iterator iterator = _threadList.iterator(); iterator.hasNext();)
        {
            TaskRunner taskRunner = (TaskRunner) iterator.next();
            _logger.debug("Stopping task runner " + taskRunner.getName());
            taskRunner.stopInvoking();
        }

        _threadGroup = null;
        _threadList.clear();

        _running = false;

        _logger.info("Scheduler stopped");
    }

    /** {@inheritDoc} */
    public void start()
    {
        if (_running)
        {
            throw new IllegalStateException("Already started");
        }

        if (_logger == null)
        {
            throw new IllegalStateException("no log");
        }

        if (_taskRunnerFactory == null)
        {
            throw new IllegalStateException("no taskRunnerFactory");
        }

        _logger.info("Starting scheduler");

        _threadGroup = new ThreadGroup("Basher Threads");

        _threadList = new ArrayList();

        for (int i = 0; i < _numberInitialThreads; i++)
        {
            addNewThread();
        }

        _running = true;

        _timedTaskInvoker.start();

        _logger.info("Scheduler started.  " + _numberInitialThreads + " thread(s) running");

    }

    /** {@inheritDoc} */
    public int getNumberOfActiveThreads()
    {
        checkInitialized();
        return _threadList.size();
    }

    /** {@inheritDoc} */
    public boolean isRunning()
    {
        return _running;
    }

    /** Adds a new thread to the currently running thread group.  Will only add a new thread if we have not yet reached
     * the maximum number of threads allowed.
     */
    private void addNewThread()
    {
        // Have we reached the maximum number of threads?
        if (_threadList.size() < _maxThreads)
        {
            // No, so add a new one
            _logger.debug("Adding new thread");

            // Create a new task runner
            final TaskRunner taskRunner = _taskRunnerFactory.createTaskRunner();

            // And the new thread to run and start it
            Thread t = new Thread(_threadGroup, taskRunner, taskRunner.getName());
            t.setDaemon(true);
            t.start();
            // Add the task runner to our internally managed list
            _threadList.add(taskRunner);
            _logger.info("Thread added.  " + _threadList.size() + " thread(s) running");
        }
        else
        {
            _logger.warn("Maximum thread limit (" + _maxThreads + ") reached, not adding more threads");
        }
    }

    /**
     * Checks if this <code>SchedulerImpl</code> instance has been started or not.
     * @throws IllegalStateException If the instance is NOT running
     */
    private void checkInitialized()
    {
        if (!_running)
        {
            throw new IllegalStateException("Not started");
        }
    }

    /**** Setters for various fields */

    public void setLog(final Log logger)
    {
        _logger = logger;
    }

    public void setMaxThreads(final int maxThreads)
    {
        _maxThreads = maxThreads;
    }

    public void setNumberInitialThreads(final int numberThreads)
    {
        _numberInitialThreads = numberThreads;
    }

    public void setTimedTaskInvoker(final TimedTaskInvoker timedTaskInvoker)
    {
        _timedTaskInvoker = timedTaskInvoker;
    }

    public void setTaskRunnerFactory(final TaskRunnerFactory taskRunnerFactory)
    {
        _taskRunnerFactory = taskRunnerFactory;
    }
}
