/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.basher.internal.impl;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import bsh.EvalError;
import bsh.Interpreter;
import net.sourceforge.basher.Scheduler;
import net.sourceforge.basher.TaskManager;
import net.sourceforge.basher.internal.BeanShellInitializer;
import org.apache.commons.logging.Log;

/**
 * @author Johan Lindquist
 */
public class BeanShellInitializerImpl implements BeanShellInitializer
{
    private String _scriptDirectory;
    private TaskManager _taskManager;
    private Log _logger;
    private Scheduler _scheduler;

    private FilenameFilter _filenameFilter;
    private Comparator _filenameComparator;

    public void setFilenameFilter(final FilenameFilter filenameFilter)
    {
        _filenameFilter = filenameFilter;
    }

    public void setFilenameComparator(final Comparator filenameComparator)
    {
        _filenameComparator = filenameComparator;
    }

    public void setLog(final Log logger)
    {
        _logger = logger;
    }

    public void setScheduler(final Scheduler scheduler)
    {
        _scheduler = scheduler;
    }

    public void setTaskManager(final TaskManager taskManager)
    {
        _taskManager = taskManager;
    }

    public void setScriptDirectory(final String scriptDirectory)
    {
        _scriptDirectory = scriptDirectory;
    }

    public void initializeService() throws Exception
    {
        // Loop over all scripts in the script directory, sorted by name
        File scriptDir = new File(_scriptDirectory);
        if (!scriptDir.exists())
        {
            throw new Exception("Script directory '" + _scriptDirectory +"' does not exist");
        }
        if (!scriptDir.canRead())
        {
            throw new Exception("Read access denied to script directory '" + _scriptDirectory +"'");
        }

        _logger.info("Using script directory " + _scriptDirectory);

        executeScripts(scriptDir);
    }

    private void executeScripts(final File scriptDir)
            throws EvalError, FileNotFoundException
    {
        File[] scriptFiles = scriptDir.listFiles(_filenameFilter);

        List scriptFileList = new ArrayList(Arrays.asList(scriptFiles));
        Collections.sort(scriptFileList, _filenameComparator);

        _logger.debug("Executing " + scriptFileList.size()+ " script(s)");

        for (Iterator iterator = scriptFileList.iterator(); iterator.hasNext();)
        {
            File scriptFile = (File) iterator.next();
            _logger.debug("Executing " + scriptFile.getName());
            Interpreter interpreter = new Interpreter();
            interpreter.set("taskManager", _taskManager);
            interpreter.set("scheduler", _scheduler);
            Reader reader = new FileReader(scriptFile);
            try
            {
                interpreter.eval(reader);
                _logger.debug(scriptFile.getName() + " executed successfully");
            }
            catch (EvalError evalError)
            {
                _logger.error(evalError.getMessage(), evalError);
            }
        }
    }

}
