/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.basher.internal.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.sourceforge.basher.internal.TimedTaskInvoker;
import net.sourceforge.basher.internal.TimedTaskRunner;
import net.sourceforge.basher.internal.TimedTaskRunnerFactory;
import org.apache.commons.logging.Log;

/**
 * @author Johan Lindquist
 * @version 1.0
 */
public class TimedTaskInvokerImpl implements TimedTaskInvoker
{
    private Log _logger;
    private List _timedTaskInfos;
    private List _timedTaskRunners;
    private TimedTaskRunnerFactory _timedTaskRunnerFactory;
    private ThreadGroup _threadGroup;
    private boolean _started = false;

    public void setTimedTaskRunnerFactory(final TimedTaskRunnerFactory timedTaskRunnerFactory)
    {
        _timedTaskRunnerFactory = timedTaskRunnerFactory;
    }

    public void setLog(final Log logger)
    {
        _logger = logger;
    }

    public void setTimedTaskInfos(List timedTaskInfos)
    {
        _timedTaskInfos = timedTaskInfos;
    }

    public boolean isRunning()
    {
        return _started;
    }

    public void start()
    {
        if (_started)
        {
            throw new IllegalStateException("Already started");
        }

        _threadGroup = new ThreadGroup("Timed Executor Thread Group");

        _timedTaskRunners = new ArrayList();

        if (_timedTaskInfos != null && !_timedTaskInfos.isEmpty())
        {
            _logger.info("Timed Task Executor Starting");
            for (Iterator iterator = _timedTaskInfos.iterator(); iterator.hasNext();)
            {
                TimedTaskInvokerInfo timedExecutorInfo = (TimedTaskInvokerInfo) iterator.next();

                if (timedExecutorInfo.getInterval()*1000 < 1000)
                {
                    _logger.warn("Timed task '" + timedExecutorInfo.getTimedTask().getName() + "' will not be run as interval is < 1 sec");
                    continue;
                }

                TimedTaskRunner timedTaskRunner = _timedTaskRunnerFactory.createTimedTaskRunner(timedExecutorInfo.getInterval(),timedExecutorInfo.getTimedTask());
                _logger.debug("Starting timed task '" + timedTaskRunner.getName() + "'");
                _timedTaskRunners.add(timedTaskRunner);
                Thread thread = new Thread(_threadGroup,timedTaskRunner);
                thread.setDaemon(true);
                thread.start();
            }
            _logger.info("Timed Task Executor started.  " + _timedTaskRunners.size() + " timed task(s) running");
        }
        else
        {
            _logger.info("No timed tasks to start");
        }
        _started = true;
    }

    public void stop()
    {
        if (!_started)
        {
            throw new IllegalStateException("Not started");
        }
        _logger.info("Stopping Timed Task Executor");
        for (Iterator iterator = _timedTaskRunners.iterator(); iterator.hasNext();)
        {
            TimedTaskRunner timedTaskRunner = (TimedTaskRunner) iterator.next();
            _logger.debug("Stopping '" + timedTaskRunner.getName() + "' timed task");
            timedTaskRunner.stopInvoking();
        }
        _logger.info("Timed Task Executor Stopped");
        _started = false;
    }

}
