/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.basher.internal.tasks;

import net.sourceforge.basher.Collector;
import net.sourceforge.basher.internal.TimeSource;
import net.sourceforge.basher.tasks.AbstractTask;
import org.apache.commons.logging.Log;

/** Task responsible for starting and stopping statistics collection before and after a certain time.
 * @author Johan Lindquist
 * @version 1.0
 */
public class CollectorMonitorTask extends AbstractTask
{
    private TimeSource _timeSource;
    private Collector _collector;
    private long _startCollectionFrom;
    private long _stopCollectionAfter;
    private Log _log;
    private boolean _finishedCollecting = false;

    public void doTask() throws Throwable
    {
        boolean collecting = _collector.isCollecting();

        _log.debug("Collecting: " + collecting);

        final long elapsedTime = _timeSource.getElapsedTime();
        if (!collecting && !_finishedCollecting)
        {
            if (_startCollectionFrom < elapsedTime)
            {
                // start collection
                _log.debug("Starting collection after " + _startCollectionFrom + " millisecond(s)");
                _collector.startCollecting();
            }
            else
            {
                _log.debug("Not collecting before " + _startCollectionFrom + " millisecond(s)");
            }
        }
        else
        {
            // We are collecting
            if (_stopCollectionAfter != 0 && _stopCollectionAfter < elapsedTime)
            {
                _log.debug("Stopping collecting after " + _stopCollectionAfter + " millisecond(s)");
                _collector.stopCollecting();
                _finishedCollecting = true;
            }
        }

    }

    public void setTimeSource(final TimeSource timeSource)
    {
        _timeSource = timeSource;
    }

    /**
     * Sets the time (in seconds) from which collection should be started.
     *
     * @param startCollectionFrom Time (in seconds) from which collection should start.
     */
    public void setStartCollectionFrom(final long startCollectionFrom)
    {
        _startCollectionFrom = startCollectionFrom * 1000;
    }

    /**
     * Sets the time (in seconds) after which collection should be stop.
     *
     * @param stopCollectionAfter Time (in seconds) after which collection should stop.
     */
    public void setStopCollectionAfter(final long stopCollectionAfter)
    {
        _stopCollectionAfter = stopCollectionAfter * 1000;
    }


    public void setLog(final Log log)
    {
        _log = log;
    }

    public void setCollector(final Collector collector)
    {
        _collector = collector;
    }

}
